@@ -46,6 +46,35 @@
 #   return(x.freq.sum)
 # }
 
+#' Follow-up times by regimen
+#'
+#' Subject specific follow-up times for each regimen in \code{wts_data}.
+#' @param wts_data Either a list of data.table containing weights (one for each separate regimen/intervention) or a single data.table with
+#' weights for one regimen / intervention.
+#' @param IDnode Name of the column containing subject-specific identifier in the input data.
+#' @param tnode Name of the column containing the time/period variable in the input data.
+#' @return A \code{data.table} of subject specific follow-up times for each regimen in \code{wts_data}.
+#' @seealso \code{\link{getIPWeights}} for evaluation of IP-weights.
+#' @export
+get_FUPtimes <- function(wts_data, IDnode, tnode) {
+  wts_data <- format_wts_data(wts_data)
+  t.name.col <- tnode
+  ID.name.col <- IDnode
+  follow_up_rule_ID <- wts_data[cum.IPAW > 0, list(max.t = max(get(t.name.col), na.rm = TRUE)), by = list(get(ID.name.col), get("rule.name"))]
+  data.table::setnames(follow_up_rule_ID, c(IDnode, "rule.name", "max.t"))
+  data.table::setkeyv(follow_up_rule_ID, cols = IDnode)
+  return(follow_up_rule_ID)
+  # rules <- unique(follow_up_rule_ID[["rule.name"]])
+  # # for (T.rule in rules) {
+  # #   one_ruleID <- follow_up_rule_ID[(rule.name %in% eval(T.rule)), max.t]
+  # #   hist(one_ruleID, main = "Maximum follow-up period for TRT/MONITOR rule: " %+% T.rule)
+  # # }
+  # T.rule <- rules[1]
+  # one_ruleID <- follow_up_rule_ID[(rule.name %in% eval(T.rule)), max.t]
+  # hist(one_ruleID, main = "Maximum follow-up period for TRT/MONITOR rule: " %+% T.rule, plot = FALSE)
+  # summary(one_ruleID)
+}
+
 #' IP-Weights Summary Tables
 #'
 #' Produces various table summaries of IP-Weights.
@@ -133,13 +162,14 @@ get_MSM_RDs <- function(MSM, t.periods.RDs, getSEs = TRUE) {
   RDs.IPAW.tperiods <- vector(mode = "list", length = length(t.periods.RDs))
   periods_idx <- seq_along(MSM$periods)
   names(RDs.IPAW.tperiods) <- "RDs_for_t" %+% t.periods.RDs
-  for (t.idx in seq(t.periods.RDs)) {
+  for (t.idx in seq_along(t.periods.RDs)) {
     t.period.val.idx <- periods_idx[MSM$periods %in% t.periods.RDs[t.idx]]
     se.RDscale.Sdt.K <- getSE_table_d_by_d(MSM$St, MSM$IC.Var.S.d, MSM$nID, t.period.val.idx, getSEs)
     RDs.IPAW.tperiods[[t.idx]] <- make.table.m0(MSM$St,
                                                 RDscale = TRUE,
                                                 t.period = t.period.val.idx,
-                                                nobs = nrow(MSM$wts_data),
+                                                nobs = MSM$nobs,
+                                                # nobs = nrow(MSM$wts_data),
                                                 esti = MSM$est_name,
                                                 se.RDscale.Sdt.K = se.RDscale.Sdt.K)
   }
@@ -170,7 +200,7 @@ get_TMLE_RDs <- function(TMLE_list, t.periods.RDs) {
   RDs.TMLE.tperiods <- vector(mode = "list", length = length(t.periods.RDs))
   periods_idx <- seq_along(new_TMLE_list$periods)
   names(RDs.TMLE.tperiods) <- "RDs_for_t" %+% t.periods.RDs
-  for (t.idx in seq(t.periods.RDs)) {
+  for (t.idx in seq_along(t.periods.RDs)) {
     t.period.val.idx <- periods_idx[new_TMLE_list$periods %in% t.periods.RDs[t.idx]]
     se.RDscale.Sdt.K <- getSE_table_d_by_d(new_TMLE_list$St, new_TMLE_list$IC.Var.S.d, new_TMLE_list$nID, t.period.val.idx, getSEs = TRUE)
     RDs.TMLE.tperiods[[t.idx]] <- make.table.m0(new_TMLE_list$St,
