@@ -12,13 +12,7 @@
 # new.pkgs <- setdiff(pkgs, rownames(installed.packages()))
 # if (length(new.pkgs)) install.packages(new.pkgs)
 ## Download and install the H2O package for R:
-# install.packages("h2o", type="source", repos=(c("https://s3.amazonaws.com/h2o-release/h2o/master/3636/R")))
-## --------------------------------------------------------------------------------------------------------
-## Install h2oEnsemble (most recent stable version 1.8)
-## --------------------------------------------------------------------------------------------------------
-# install.packages("https://h2o-release.s3.amazonaws.com/h2o-ensemble/R/h2oEnsemble_0.1.8.tar.gz", repos = NULL)
-# # Install h2oEnsemble (dev version):
-# devtools::install_github("h2oai/h2o-3/h2o-r/ensemble/h2oEnsemble-package")
+# install.packages("h2o", type="source", repos=(c("https://s3.amazonaws.com/h2o-release/h2o/master/3687/R")))
 ## --------------------------------------------------------------------------------------------------------
 ## Install stremr
 ## --------------------------------------------------------------------------------------------------------
@@ -141,7 +135,6 @@ test.speedglm.allestimators10Kdata <- function() {
   # ------------------------------------------------------------------------
   # require("doParallel")
   # registerDoParallel(cores = 2)
-  if (exists("setthreads")) data.table::setthreads(1)
 
   t.surv <- c(0,1,4)
   Qforms <- rep.int("Q.kplus1 ~ CVD + highA1c + N + lastNat1 + TI + TI.tminus1", (max(t.surv)+1))
@@ -178,13 +171,65 @@ test.speedglm.allestimators10Kdata <- function() {
 
   if (rmarkdown::pandoc_available(version = "1.12.3"))
     make_report_rmd(OData, NPMSM = list(surv1, surv2), MSM = MSM.IPAW, GCOMP = list(gcomp_est1, gcomp_est2), TMLE = list(tmle_est_par1, tmle_est_par2),
+                  format = "html",
+                  FUPtables = get_FUPtimes(MSM.IPAW$wts_data, IDnode = "ID", tnode = "t"),
+                  # openFile = TRUE,
+                  openFile = FALSE,
+                  MSM.RDtables = get_MSM_RDs(MSM.IPAW, t.periods.RDs = c(12, 15), getSEs = TRUE),
+                  TMLE.RDtables = get_TMLE_RDs(list(tmle_est_par1, tmle_est_par2), t.periods.RDs = c(1, 4)),
+                  WTtables = get_wtsummary(MSM.IPAW$wts_data, cutoffs = c(0, 0.5, 1, 10, 20, 30, 40, 50, 100, 150), by.rule = TRUE),
+                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name", x_legend = 9.5, y_legend = 0.99,
+                  save_report_data = TRUE
+                  )
+
+  load(file = file.path(getOption('stremr.file.path'), "sim.data.example.fup") %+% ".Rd")
+  names(report_results_list)
+
+  if (rmarkdown::pandoc_available(version = "1.12.3"))
+    make_report_rmd(OData, NPMSM = report_results_list$NPMSM, MSM = report_results_list$MSM, GCOMP = report_results_list$GCOMP, TMLE = report_results_list$TMLE,
+                  format = "html",
+                  FUPtables = report_results_list$FUPtables,
+                  # openFile = TRUE,
+                  openFile = FALSE,
+                  MSM.RDtables = report_results_list$MSM.RDtables,
+                  TMLE.RDtables = report_results_list$TMLE.RDtables,
+                  WTtables = report_results_list$WTtables,
+                  file.name = "sim.data.example.fup",
+                  title = "Custom Report Title", author = "Insert Author Name", x_legend = 9.5, y_legend = 0.99
+                  )
+
+  if (rmarkdown::pandoc_available(version = "1.12.3"))
+    make_report_rmd(OData, NPMSM = list(surv1, surv2), MSM = MSM.IPAW, GCOMP = list(gcomp_est1, gcomp_est2), TMLE = list(tmle_est_par1, tmle_est_par2),
+                  format = "html",
+                  AddFUPtables = TRUE,
+                  # openFile = TRUE,
+                  openFile = FALSE,
+                  MSM.RDtables = get_MSM_RDs(MSM.IPAW, t.periods.RDs = c(12, 15), getSEs = TRUE),
+                  TMLE.RDtables = get_TMLE_RDs(list(tmle_est_par1, tmle_est_par2), t.periods.RDs = c(1, 4)),
+                  WTtables = get_wtsummary(MSM.IPAW$wts_data, cutoffs = c(0, 0.5, 1, 10, 20, 30, 40, 50, 100, 150), by.rule = TRUE),
+                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name")
+
+  if (rmarkdown::pandoc_available(version = "1.12.3"))
+    make_report_rmd(OData, NPMSM = list(surv1, surv2), MSM = MSM.IPAW, GCOMP = list(gcomp_est1, gcomp_est2), TMLE = list(tmle_est_par1, tmle_est_par2),
+                  format = "html",
+                  AddFUPtables = TRUE,
+                  # openFile = TRUE,
+                  openFile = FALSE,
+                  MSM.RDtables = get_MSM_RDs(MSM.IPAW, t.periods.RDs = c(12, 15), getSEs = TRUE),
+                  TMLE.RDtables = get_TMLE_RDs(list(tmle_est_par1, tmle_est_par2), t.periods.RDs = c(1, 4)),
+                  WTtables = get_wtsummary(MSM.IPAW$wts_data, cutoffs = c(0, 0.5, 1, 10, 20, 30, 40, 50, 100, 150), by.rule = TRUE),
+                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name", x_legend = "topright")
+
+
+  if (rmarkdown::pandoc_available(version = "1.12.3"))
+    make_report_rmd(OData, NPMSM = list(surv1, surv2), plotKM = TRUE, MSM = MSM.IPAW, GCOMP = list(gcomp_est1, gcomp_est2), TMLE = list(tmle_est_par1, tmle_est_par2),
                   format = "html",
                   AddFUPtables = TRUE,
                   openFile = FALSE,
                   MSM.RDtables = get_MSM_RDs(MSM.IPAW, t.periods.RDs = c(12, 15), getSEs = TRUE),
                   TMLE.RDtables = get_TMLE_RDs(list(tmle_est_par1, tmle_est_par2), t.periods.RDs = c(1, 4)),
                   WTtables = get_wtsummary(MSM.IPAW$wts_data, cutoffs = c(0, 0.5, 1, 10, 20, 30, 40, 50, 100, 150), by.rule = TRUE),
-                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name", y_legend = 0.99, x_legend = 9.5)
+                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name", x_legend = 9.5, y_legend = 0.99)
 
   if (rmarkdown::pandoc_available(version = "1.12.3"))
     make_report_rmd(OData, NPMSM = list(surv1, surv2), MSM = MSM.IPAW, TMLE = list(tmle_est_par1, tmle_est_par2),
@@ -194,7 +239,7 @@ test.speedglm.allestimators10Kdata <- function() {
                   MSM.RDtables = get_MSM_RDs(MSM.IPAW, t.periods.RDs = c(12, 15), getSEs = TRUE),
                   TMLE.RDtables = get_TMLE_RDs(list(tmle_est_par1, tmle_est_par2), t.periods.RDs = c(1, 4)),
                   WTtables = get_wtsummary(MSM.IPAW$wts_data, cutoffs = c(0, 0.5, 1, 10, 20, 30, 40, 50, 100, 150), by.rule = TRUE),
-                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name", y_legend = 0.99, x_legend = 9.5)
+                  file.name = "sim.data.example.fup", title = "Custom Report Title", author = "Insert Author Name", x_legend = 9.5, y_legend = 0.99)
 
   # ---------------------------------------------------------------------------------------------------------
   # TMLE / GCOMP with a stochastic intervention on MONITOR
